@@ -83,6 +83,8 @@ three extensions points (all of them optional):
       maxunicode |0xffff
 """
 
+skip_doctest = True  # ignore top-level docstring as a doctest.
+
 import sys, os, os.path, stat, glob, new, csv, datetime, types
 import itertools, mimetypes, StringIO
 
@@ -123,8 +125,7 @@ except ImportError:
     grp = None
 
 from IPython.external import simplegeneric
-
-import path
+from IPython.external import path
 
 try:
     from IPython import genutils, generics
@@ -1210,8 +1211,12 @@ class ils(Table):
     Examples::
 
         >>> ils
+        <class 'IPython.Extensions.ipipe.ils'>
         >>> ils("/usr/local/lib/python2.4")
+        IPython.Extensions.ipipe.ils('/usr/local/lib/python2.4')
         >>> ils("~")
+        IPython.Extensions.ipipe.ils('/home/fperez')
+        # all-random
     """
     def __init__(self, base=os.curdir, dirs=True, files=True):
         self.base = os.path.expanduser(base)
@@ -1248,6 +1253,7 @@ class iglob(Table):
     Examples::
 
         >>> iglob("*.py")
+        IPython.Extensions.ipipe.iglob('*.py')
     """
     def __init__(self, glob):
         self.glob = glob
@@ -1273,8 +1279,12 @@ class iwalk(Table):
     List all files and directories in a directory and it's subdirectory::
 
         >>> iwalk
-        >>> iwalk("/usr/local/lib/python2.4")
+        <class 'IPython.Extensions.ipipe.iwalk'>
+        >>> iwalk("/usr/lib")
+        IPython.Extensions.ipipe.iwalk('/usr/lib')
         >>> iwalk("~")
+        IPython.Extensions.ipipe.iwalk('/home/fperez')  # random
+        
     """
     def __init__(self, base=os.curdir, dirs=True, files=True):
         self.base = os.path.expanduser(base)
@@ -1378,6 +1388,8 @@ class ipwd(Table):
     Example::
 
         >>> ipwd | isort("uid")
+        <IPython.Extensions.ipipe.isort key='uid' reverse=False at 0x849efec>
+        # random
     """
     def __iter__(self):
         for entry in pwd.getpwall():
@@ -1562,6 +1574,7 @@ class ienv(Table):
     Example::
 
         >>> ienv
+        <class 'IPython.Extensions.ipipe.ienv'>
     """
 
     def __iter__(self):
@@ -1583,7 +1596,9 @@ class ihist(Table):
     Example::
 
         >>> ihist
-        >>> ihist(True) (raw mode)
+        <class 'IPython.Extensions.ipipe.ihist'>
+        >>> ihist(True) # raw mode
+        <IPython.Extensions.ipipe.ihist object at 0x849602c>  # random
     """
     def __init__(self, raw=True):
         self.raw = raw
@@ -1618,6 +1633,7 @@ class ialias(Table):
     Example::
 
         >>> ialias
+        <class 'IPython.Extensions.ipipe.ialias'>
     """
     def __iter__(self):
         api = ipapi.get()
@@ -1680,7 +1696,11 @@ class ix(Table):
     Examples::
 
         >>> ix("ps x")
+        IPython.Extensions.ipipe.ix('ps x')
+
         >>> ix("find .") | ifile
+        <IPython.Extensions.ipipe.ieval expr=<class 'IPython.Extensions.ipipe.ifile'> at 0x8509d2c>
+        # random
     """
     def __init__(self, cmd):
         self.cmd = cmd
@@ -1721,6 +1741,7 @@ class ifilter(Pipe):
         >>> ils | ifilter("_.isfile() and size>1000")
         >>> igrp | ifilter("len(mem)")
         >>> sys.modules | ifilter(lambda _:_.value is not None)
+        # all-random
     """
 
     def __init__(self, expr, globals=None, errors="raiseifallfail"):
@@ -1811,7 +1832,9 @@ class ieval(Pipe):
     Examples::
 
         >>> ils | ieval("_.abspath()")
+        # random
         >>> sys.path | ieval(ifile)
+        # random
     """
 
     def __init__(self, expr, globals=None, errors="raiseifallfail"):
@@ -1884,6 +1907,8 @@ class ienum(Pipe):
 
         >>> xrange(20) | ieval("_,_*_") | ienum | ifilter("index % 2 == 0") | ieval("object")
     """
+    skip_doctest = True
+    
     def __iter__(self):
         fields = ("index", "object")
         for (index, object) in enumerate(xiter(self.input)):
@@ -1897,7 +1922,10 @@ class isort(Pipe):
     Examples::
 
         >>> ils | isort("size")
+        <IPython.Extensions.ipipe.isort key='size' reverse=False at 0x849ec2c>
         >>> ils | isort("_.isdir(), _.lower()", reverse=True)
+        <IPython.Extensions.ipipe.isort key='_.isdir(), _.lower()' reverse=True at 0x849eacc>
+        # all-random
     """
 
     def __init__(self, key=None, globals=None, reverse=False):
@@ -2058,6 +2086,8 @@ class icap(Table):
         >>> icap("for i in range(10): print i, time.sleep(0.1)")
 
     """
+    skip_doctest = True
+    
     def __init__(self, expr, globals=None):
         self.expr = expr
         self.globals = globals
