@@ -5,7 +5,7 @@ General purpose utilities.
 This is a grab-bag of stuff I find useful in most programs I write. Some of
 these things are also convenient when working at the command line.
 
-$Id: genutils.py 2872 2007-11-25 17:58:05Z fperez $"""
+$Id: genutils.py 2998 2008-01-31 10:06:04Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -22,26 +22,50 @@ __license__ = Release.license
 # required modules from the Python standard library
 import __main__
 import commands
-import doctest
+try:
+    import doctest
+except ImportError:
+    pass
 import os
+import platform
 import re
 import shlex
 import shutil
+import subprocess
 import sys
 import tempfile
 import time
 import types
 import warnings
 
+# Curses and termios are Unix-only modules
+try:
+    import curses
+    # We need termios as well, so if its import happens to raise, we bail on
+    # using curses altogether.
+    import termios
+except ImportError:
+    USE_CURSES = False
+else:
+    # Curses on Solaris may not be complete, so we can't use it there
+    USE_CURSES = hasattr(curses,'initscr')
+
 # Other IPython utilities
 import IPython
 from IPython.Itpl import Itpl,itpl,printpl
 from IPython import DPyGetOpt, platutils
 from IPython.generics import result_display
-from path import path
+import IPython.ipapi
+from IPython.external.path import path
 if os.name == "nt":
     from IPython.winconsole import get_console_size
 
+try:
+    set
+except:
+    from sets import Set as set
+
+
 #****************************************************************************
 # Exceptions
 class Error(Exception):
@@ -73,8 +97,8 @@ class IOStream:
 
     def close(self):
         pass
-                
-        
+
+
 class IOTerm:
     """ Term holds the file or file-like objects for handling I/O operations.
 
@@ -89,16 +113,16 @@ class IOTerm:
         self.cin  = IOStream(cin,sys.stdin)
         self.cout = IOStream(cout,sys.stdout)
         self.cerr = IOStream(cerr,sys.stderr)
-        
+
 # Global variable to be used for all I/O
 Term = IOTerm()
 
 import IPython.rlineimpl as readline
 # Remake Term to use the readline i/o facilities
 if sys.platform == 'win32' and readline.have_readline:
-    
+
     Term = IOTerm(cout=readline._outputfile,cerr=readline._outputfile)
-    
+
 
 #****************************************************************************
 # Generic warning/error printer, used by everything else
@@ -106,9 +130,9 @@ def warn(msg,level=2,exit_val=1):
     """Standard warning printer. Gives formatting consistency.
 
     Output is sent to Term.cerr (sys.stderr by default).
-    
+
     Options:
-    
+
     -level(2): allows finer control:
       0 -> Do nothing, dummy function.
       1 -> Print message.
@@ -118,7 +142,7 @@ def warn(msg,level=2,exit_val=1):
 
     -exit_val (1): exit value returned by sys.exit() for a level 4
     warning. Ignored for all other levels."""
-    
+
     if level>0:
         header = ['','','WARNING: ','ERROR: ','FATAL ERROR: ']
         print >> Term.cerr, '%s%s' % (header[level],msg)
@@ -143,10 +167,10 @@ def fatal(msg,exit_val=1):
 
 #---------------------------------------------------------------------------
 # Debugging routines
-# 
+#
 def debugx(expr,pre_msg=''):
     """Print the value of an expression from the caller's frame.
-    
+
     Takes an expression, evaluates it in the caller's frame and prints both
     the given expression and the resulting value (as well as a debug mark
     indicating the name of the calling function.  The input must be of a form
@@ -154,7 +178,7 @@ def debugx(expr,pre_msg=''):
 
     An optional message can be passed, which will be prepended to the printed
     expr->value pair."""
-    
+
     cf = sys._getframe(1)
     print '[DBG:%s] %s%s -> %r' % (cf.f_code.co_name,pre_msg,expr,
                                    eval(expr,cf.f_globals,cf.f_locals))
@@ -176,18 +200,18 @@ try:
         Return the *USER* CPU time in seconds since the start of the process.
         This is done via a call to resource.getrusage, so it avoids the
         wraparound problems in time.clock()."""
-        
+
         return resource.getrusage(resource.RUSAGE_SELF)[0]
-    
+
     def clocks():
         """clocks() -> floating point number
 
         Return the *SYSTEM* CPU time in seconds since the start of the process.
         This is done via a call to resource.getrusage, so it avoids the
         wraparound problems in time.clock()."""
-        
+
         return resource.getrusage(resource.RUSAGE_SELF)[1]
-    
+
     def clock():
         """clock() -> floating point number
 
@@ -195,9 +219,9 @@ try:
         the process.  This is done via a call to resource.getrusage, so it
         avoids the wraparound problems in time.clock()."""
 
-        u,s = resource.getrusage(resource.RUSAGE_SELF)[:2] 
+        u,s = resource.getrusage(resource.RUSAGE_SELF)[:2]
         return u+s
-    
+
     def clock2():
         """clock2() -> (t_user,t_system)
 
@@ -223,7 +247,7 @@ def timings_out(reps,func,*args,**kw):
     Under Unix, the return value is the sum of user+system time consumed by
     the process, computed via the resource module.  This prevents problems
     related to the wraparound effect which the time.clock() function has.
-    
+
     Under Windows the return value is in wall clock seconds. See the
     documentation for the time module for more details."""
 
@@ -286,7 +310,7 @@ def system(cmd,verbose=0,debug=0,header=''):
     Options:
 
     - verbose (0): print the command to be executed.
-    
+
     - debug (0): only print, do not actually execute.
 
     - header (''): Header to print on screen prior to the executed command (it
@@ -303,16 +327,22 @@ def system(cmd,verbose=0,debug=0,header=''):
 
 def abbrev_cwd():
     """ Return abbreviated version of cwd, e.g. d:mydir """
-    cwd = os.getcwd()
+    cwd = os.getcwd().replace('\\','/')
     drivepart = ''
+    tail = cwd
     if sys.platform == 'win32':
         if len(cwd) < 4:
             return cwd
-        drivepart = os.path.splitdrive(cwd)[0]
+        drivepart,tail = os.path.splitdrive(cwd)
+
+
+    parts = tail.split('/')
+    if len(parts) > 2:
+        tail = '/'.join(parts[-2:])
+
     return (drivepart + (
-        cwd == '/' and '/' or \
-        os.path.basename(cwd)))
-    
+        cwd == '/' and '/' or tail))
+
 
 # This function is used by ipython in a lot of places to make system calls.
 # We need it to be slightly different under win32, due to the vagaries of
@@ -324,7 +354,7 @@ def shell(cmd,verbose=0,debug=0,header=''):
     Options:
 
     - verbose (0): print the command to be executed.
-    
+
     - debug (0): only print, do not actually execute.
 
     - header (''): Header to print on screen prior to the executed command (it
@@ -338,7 +368,7 @@ def shell(cmd,verbose=0,debug=0,header=''):
     if verbose or debug: print header+cmd
     # flush stdout so we don't mangle python's buffering
     sys.stdout.flush()
-    
+
     if not debug:
         platutils.set_term_title("IPy " + cmd)
         os.system(cmd)
@@ -376,10 +406,10 @@ def getoutput(cmd,verbose=0,debug=0,header='',split=0):
 
     Note: a stateful version of this function is available through the
     SystemExec class.
-    
-    This is pretty much deprecated and rarely used, 
+
+    This is pretty much deprecated and rarely used,
     genutils.getoutputerror may be what you need.
-    
+
     """
 
     if verbose or debug: print header+cmd
@@ -431,7 +461,7 @@ class SystemExec:
 
     Note: here we refer to the system and getoutput functions from this
     library, not the ones from the standard python library.
-    
+
     This class offers the system and getoutput functions as methods, but the
     verbose, debug and header parameters can be set for the instance (at
     creation time or later) so that they don't need to be specified on each
@@ -446,13 +476,9 @@ class SystemExec:
      - bq: alias to getoutput
 
     An instance can then be created as:
-     >>> sysexec = SystemExec(verbose=1,debug=0,header='Calling: ')
-
-    And used as:
-     >>> sysexec.xsys('pwd')
-     >>> dirlist = sysexec.bq('ls -l')
+    >>> sysexec = SystemExec(verbose=1,debug=0,header='Calling: ')
     """
-    
+
     def __init__(self,verbose=0,debug=0,header='',split=0):
         """Specify the instance's values for verbose, debug and header."""
         setattr_list(self,'verbose debug header split')
@@ -604,7 +630,7 @@ def process_cmdline(argv,names=[],defaults={},usage=''):
     Arguments:
 
     - argv: list of arguments, typically sys.argv.
-    
+
     - names: list of option names. See DPyGetOpt docs for details on options
     syntax.
 
@@ -626,7 +652,7 @@ def process_cmdline(argv,names=[],defaults={},usage=''):
 
     defaults.update(getopt.optionValues)
     args = getopt.freeValues
-    
+
     return defaults,args
 
 #----------------------------------------------------------------------------
@@ -654,8 +680,7 @@ def optstr2types(ostr):
 
 #----------------------------------------------------------------------------
 def read_dict(filename,type_conv=None,**opt):
-
-    """Read a dictionary of key=value pairs from an input file, optionally
+    r"""Read a dictionary of key=value pairs from an input file, optionally
     performing conversions on the resulting values.
 
     read_dict(filename,type_conv,**opt) -> dict
@@ -701,20 +726,33 @@ def read_dict(filename,type_conv=None,**opt):
         to make a list of all appearances.
 
     Example:
-    If the input file test.ini has:
-      i 3
-      x 4.5
-      y 5.5
-      s hi ho
-    Then:
 
+    If the input file test.ini contains (we put it in a string to keep the test
+    self-contained):
+
+    >>> test_ini = '''\
+    ... i 3
+    ... x 4.5
+    ... y 5.5
+    ... s hi ho'''
+
+    Then we can use it as follows:
     >>> type_conv={int:'i',float:'x',None:'s'}
-    >>> read_dict('test.ini')
-    {'i': '3', 's': 'hi ho', 'x': '4.5', 'y': '5.5'}
-    >>> read_dict('test.ini',type_conv)
-    {'i': 3, 's': 'hi ho', 'x': 4.5, 'y': '5.5'}
-    >>> read_dict('test.ini',type_conv,purge=1)
-    {'i': 3, 's': 'hi ho', 'x': 4.5}
+
+    >>> d = read_dict(test_ini)
+
+    >>> sorted(d.items())
+    [('i', '3'), ('s', 'hi ho'), ('x', '4.5'), ('y', '5.5')]
+
+    >>> d = read_dict(test_ini,type_conv)
+
+    >>> sorted(d.items())
+    [('i', 3), ('s', 'hi ho'), ('x', 4.5), ('y', '5.5')]
+
+    >>> d = read_dict(test_ini,type_conv,purge=True)
+
+    >>> sorted(d.items())
+    [('i', 3), ('s', 'hi ho'), ('x', 4.5)]
     """
 
     # starting config
@@ -732,9 +770,15 @@ def read_dict(filename,type_conv=None,**opt):
         raise ValueError, 'Unique keys must be given as a string, List or Tuple'
 
     dict = {}
+
     # first read in table of values as strings
-    file = open(filename,'r')
-    for line in file.readlines():
+    if '\n' in filename:
+        lines = filename.splitlines()
+        file = None
+    else:
+        file = open(filename,'r')
+        lines = file.readlines()
+    for line in lines:
         line = line.strip()
         if len(line) and line[0]=='#': continue
         if len(line)>0:
@@ -801,7 +845,7 @@ def flag_calls(func):
 
     Testing for truth in wrapper.called allows you to determine if a call to
     func() was attempted and succeeded."""
-    
+
     def wrapper(*args,**kw):
         wrapper.called = False
         out = func(*args,**kw)
@@ -823,7 +867,7 @@ def dhook_wrap(func,*a,**k):
     """
 
     def f(*a,**k):
-    
+
         dhook_s = sys.displayhook
         sys.displayhook = sys.__displayhook__
         try:
@@ -843,10 +887,10 @@ def doctest_reload():
     This routine:
 
       - reloads doctest
-      
+
       - resets its global 'master' attribute to None, so that multiple uses of
       the module interactively don't produce cumulative reports.
-    
+
       - Monkeypatches its core test runner method to protect it from IPython's
       modified displayhook.  Doctest expects the default displayhook behavior
       deep down, so our modification breaks it completely.  For this reason, a
@@ -880,16 +924,16 @@ def get_home_dir():
 
     isdir = os.path.isdir
     env = os.environ
-    
+
     # first, check py2exe distribution root directory for _ipython.
     # This overrides all. Normally does not exist.
-    
+
     if '\\library.zip\\' in IPython.__file__.lower():
         root, rest = IPython.__file__.lower().split('library.zip')
         if isdir(root + '_ipython'):
             os.environ["IPYKITROOT"] = root.rstrip('\\')
             return root
-    
+
     try:
         homedir = env['HOME']
         if not isdir(homedir):
@@ -935,6 +979,38 @@ def get_home_dir():
         else:
             raise HomeDirError,'support for your operating system not implemented.'
 
+
+def get_ipython_dir():
+    """Get the IPython directory for this platform and user.
+    
+    This uses the logic in `get_home_dir` to find the home directory
+    and the adds either .ipython or _ipython to the end of the path.
+    """
+    if os.name == 'posix':
+         ipdir_def = '.ipython'
+    else:
+         ipdir_def = '_ipython'
+    home_dir = get_home_dir()
+    ipdir = os.path.abspath(os.environ.get('IPYTHONDIR',
+                                           os.path.join(home_dir,ipdir_def)))
+    return ipdir
+
+def get_security_dir():
+    """Get the IPython security directory.
+    
+    This directory is the default location for all security related files,
+    including SSL/TLS certificates and FURL files.
+    
+    If the directory does not exist, it is created with 0700 permissions.
+    If it exists, permissions are set to 0700.
+    """
+    security_dir = os.path.join(get_ipython_dir(), 'security')
+    if not os.path.isdir(security_dir):
+        os.mkdir(security_dir, 0700)
+    else:
+        os.chmod(security_dir, 0700)
+    return security_dir
+        
 #****************************************************************************
 # strings and text
 
@@ -947,7 +1023,7 @@ class LSString(str):
         .n (or .nlstr): original value (the string itself).
         .s (or .spstr): value as whitespace-separated string.
         .p (or .paths): list of path objects
-        
+
     Any values which require transformations are computed only once and
     cached.
 
@@ -983,14 +1059,14 @@ class LSString(str):
         except AttributeError:
             self.__paths = [path(p) for p in self.split('\n') if os.path.exists(p)]
             return self.__paths
-    
+
     p = paths = property(get_paths)
 
 def print_lsstring(arg):
     """ Prettier (non-repr-like) and more informative printer for LSString """
     print "LSString (.p, .n, .l, .s available). Value:"
     print arg
-    
+
 print_lsstring = result_display.when_type(LSString)(print_lsstring)
 
 #----------------------------------------------------------------------------
@@ -1003,7 +1079,7 @@ class SList(list):
         .n (or .nlstr): value as a string, joined on newlines.
         .s (or .spstr): value as a string, joined on spaces.
         .p (or .paths): list of path objects
-        
+
     Any values which require transformations are computed only once and
     cached."""
 
@@ -1029,55 +1105,71 @@ class SList(list):
             return self.__nlstr
 
     n = nlstr = property(get_nlstr)
-    
+
     def get_paths(self):
         try:
             return self.__paths
         except AttributeError:
             self.__paths = [path(p) for p in self if os.path.exists(p)]
             return self.__paths
-    
+
     p = paths = property(get_paths)
 
-    def grep(self, pattern, prune = False):
-        """ Return all strings matching 'pattern' (a regex or callable) 
-        
+    def grep(self, pattern, prune = False, field = None):
+        """ Return all strings matching 'pattern' (a regex or callable)
+
         This is case-insensitive. If prune is true, return all items
         NOT matching the pattern.
-        
+
+        If field is specified, the match must occur in the specified
+        whitespace-separated field.
+
         Examples::
-        
+
             a.grep( lambda x: x.startswith('C') )
             a.grep('Cha.*log', prune=1)
+            a.grep('chm', field=-1)
         """
+
+        def match_target(s):
+            if field is None:
+                return s
+            parts = s.split()
+            try:
+                tgt = parts[field]
+                return tgt
+            except IndexError:
+                return ""
+
         if isinstance(pattern, basestring):
             pred = lambda x : re.search(pattern, x, re.IGNORECASE)
         else:
             pred = pattern
         if not prune:
-            return SList([el for el in self if pred(el)])
+            return SList([el for el in self if pred(match_target(el))])
         else:
-            return SList([el for el in self if not pred(el)])
+            return SList([el for el in self if not pred(match_target(el))])
     def fields(self, *fields):
         """ Collect whitespace-separated fields from string list
-        
+
         Allows quick awk-like usage of string lists.
-        
+
         Example data (in var a, created by 'a = !ls -l')::
             -rwxrwxrwx  1 ville None      18 Dec 14  2006 ChangeLog
-            drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython        
-        
+            drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython
+
         a.fields(0) is ['-rwxrwxrwx', 'drwxrwxrwx+']
         a.fields(1,0) is ['1 -rwxrwxrwx', '6 drwxrwxrwx+']
         (note the joining by space).
-        
+        a.fields(-1) is ['ChangeLog', 'IPython']
+
         IndexErrors are ignored.
-        
+
         Without args, fields() just split()'s the strings.
         """
         if len(fields) == 0:
             return [el.split() for el in self]
-        
+
         res = SList()
         for el in [f.split() for f in self]:
             lineparts = []
@@ -1089,18 +1181,45 @@ class SList(list):
                     pass
             if lineparts:
                 res.append(" ".join(lineparts))
-            
-        return res        
-                
-            
-            
-        
+
+        return res
+    def sort(self,field= None,  nums = False):
+        """ sort by specified fields (see fields())
+
+        Example::
+            a.sort(1, nums = True)
+
+        Sorts a by second field, in numerical order (so that 21 > 3)
+
+        """
+
+        #decorate, sort, undecorate
+        if field is not None:
+            dsu = [[SList([line]).fields(field),  line] for line in self]
+        else:
+            dsu = [[line,  line] for line in self]
+        if nums:
+            for i in range(len(dsu)):
+                numstr = "".join([ch for ch in dsu[i][0] if ch.isdigit()])
+                try:
+                    n = int(numstr)
+                except ValueError:
+                    n = 0;
+                dsu[i][0] = n
+
+
+        dsu.sort()
+        return SList([t[1] for t in dsu])
 
 def print_slist(arg):
     """ Prettier (non-repr-like) and more informative printer for SList """
-    print "SList (.p, .n, .l, .s, .grep(), .fields() available). Value:"
+    print "SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):"
+    if hasattr(arg,  'hideonce') and arg.hideonce:
+        arg.hideonce = False
+        return
+
     nlprint(arg)
-    
+
 print_slist = result_display.when_type(SList)(print_slist)
 
 
@@ -1114,14 +1233,14 @@ def esc_quotes(strng):
 #----------------------------------------------------------------------------
 def make_quoted_expr(s):
     """Return string s in appropriate quotes, using raw string if possible.
-    
+
     Effectively this turns string: cd \ao\ao\
     to: r"cd \ao\ao\_"[:-1]
-    
+
     Note the use of raw string and padding at the end to allow trailing backslash.
-    
+
     """
-    
+
     tail = ''
     tailpadding = ''
     raw  = ''
@@ -1172,7 +1291,7 @@ def raw_input_multi(header='', ps1='==> ', ps2='..> ',terminate_str = '.'):
             while new_line.endswith('\\'):
                 new_line = new_line[:-1] + raw_input(ps2)
             lines.append(new_line)
-                
+
         return lines[:-1]  # don't return the termination command
     except EOFError:
         print
@@ -1214,7 +1333,7 @@ def ask_yes_no(prompt,default=None):
                 print
             else:
                 raise
-            
+
     return answers[ans]
 
 #----------------------------------------------------------------------------
@@ -1233,9 +1352,12 @@ class EvalDict:
     Emulate a dict which evaluates its contents in the caller's frame.
 
     Usage:
-    >>>number = 19
-    >>>text = "python"
-    >>>print "%(text.capitalize())s %(number/9.0).1f rules!" % EvalDict()
+    >>> number = 19
+
+    >>> text = "python"
+
+    >>> print "%(text.capitalize())s %(number/9.0).1f rules!" % EvalDict()
+    Python 2.1 rules!
     """
 
     # This version is due to sismex01@hebmex.com on c.l.py, and is basically a
@@ -1255,14 +1377,19 @@ def qw(words,flat=0,sep=None,maxsplit=-1):
     qw(words,flat=0,sep=' ',maxsplit=-1) -> words.split(sep,maxsplit)
 
     words can also be a list itself, and with flat=1, the output will be
-    recursively flattened. Examples:
+    recursively flattened.
+
+    Examples:
 
     >>> qw('1 2')
     ['1', '2']
+
     >>> qw(['a b','1 2',['m n','p q']])
     [['a', 'b'], ['1', '2'], [['m', 'n'], ['p', 'q']]]
+
     >>> qw(['a b','1 2',['m n','p q']],flat=1)
-    ['a', 'b', '1', '2', 'm', 'n', 'p', 'q']    """
+    ['a', 'b', '1', '2', 'm', 'n', 'p', 'q']
+    """
 
     if type(words) in StringTypes:
         return [word.strip() for word in words.split(sep,maxsplit)
@@ -1342,7 +1469,7 @@ def igrep(pat,list):
 #----------------------------------------------------------------------------
 def indent(str,nspaces=4,ntabs=0):
     """Indent a string a given number of spaces or tabstops.
-    
+
     indent(str,nspaces=4,ntabs=0) -> indent str by ntabs+nspaces.
     """
     if str is None:
@@ -1353,7 +1480,7 @@ def indent(str,nspaces=4,ntabs=0):
         return outstr[:-len(ind)]
     else:
         return outstr
-    
+
 #-----------------------------------------------------------------------------
 def native_line_ends(filename,backup=1):
     """Convert (in-place) a file to line-ends native to the current OS.
@@ -1379,13 +1506,13 @@ def native_line_ends(filename,backup=1):
             os.remove(bak_filename)
         except:
             pass
-    
+
 #----------------------------------------------------------------------------
 def get_pager_cmd(pager_cmd = None):
     """Return a pager command.
 
     Makes some attempts at finding an OS-correct one."""
-    
+
     if os.name == 'posix':
         default_pager_cmd = 'less -r'  # -r for color control sequences
     elif os.name in ['nt','dos']:
@@ -1484,7 +1611,20 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     If no system pager works, the string is sent through a 'dumb pager'
     written in python, very simplistic.
     """
-    
+
+    # Some routines may auto-compute start offsets incorrectly and pass a
+    # negative value.  Offset to 0 for robustness.
+    start = max(0,start)
+
+    # first, try the hook
+    ip = IPython.ipapi.get()
+    if ip:
+        try:
+            ip.IP.hooks.show_in_pager(strng)
+            return
+        except IPython.ipapi.TryNext:
+            pass
+
     # Ugly kludge, but calling curses.initscr() flat out crashes in emacs
     TERM = os.environ.get('TERM','dumb')
     if TERM in ['dumb','emacs'] and os.name != 'nt':
@@ -1495,7 +1635,7 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     str_toprint = os.linesep.join(str_lines)
     num_newlines = len(str_lines)
     len_str = len(str_toprint)
-    
+
     # Dumb heuristics to guesstimate number of on-screen lines the string
     # takes.  Very basic, but good enough for docstrings in reasonable
     # terminals. If someone later feels like refining it, it's not hard.
@@ -1509,26 +1649,30 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     # auto-determine screen size
     if screen_lines <= 0:
         if TERM=='xterm':
-            try:
-                import curses
-                if hasattr(curses,'initscr'):
-                    use_curses = 1
-                else:
-                    use_curses = 0
-            except ImportError:
-                use_curses = 0
+            use_curses = USE_CURSES
         else:
             # curses causes problems on many terminals other than xterm.
-            use_curses = 0
+            use_curses = False
         if use_curses:
-                scr = curses.initscr()
-                screen_lines_real,screen_cols = scr.getmaxyx()
-                curses.endwin()
-                screen_lines += screen_lines_real
-                #print '***Screen size:',screen_lines_real,'lines x',\
-                #screen_cols,'columns.' # dbg
+            # There is a bug in curses, where *sometimes* it fails to properly
+            # initialize, and then after the endwin() call is made, the
+            # terminal is left in an unusable state.  Rather than trying to
+            # check everytime for this (by requesting and comparing termios
+            # flags each time), we just save the initial terminal state and
+            # unconditionally reset it every time.  It's cheaper than making
+            # the checks.
+            term_flags = termios.tcgetattr(sys.stdout)
+            scr = curses.initscr()
+            screen_lines_real,screen_cols = scr.getmaxyx()
+            curses.endwin()
+            # Restore terminal state in case endwin() didn't.
+            termios.tcsetattr(sys.stdout,termios.TCSANOW,term_flags)
+            # Now we have what we needed: the screen size in rows/columns
+            screen_lines += screen_lines_real
+            #print '***Screen size:',screen_lines_real,'lines x',\
+            #screen_cols,'columns.' # dbg
         else:
-                screen_lines += screen_lines_def
+            screen_lines += screen_lines_def
 
     #print 'numlines',numlines,'screenlines',screen_lines  # dbg
     if numlines <= screen_lines :
@@ -1596,6 +1740,7 @@ def page_file(fname,start = 0, pager_cmd = None):
         except:
             print 'Unable to show file',`fname`
 
+
 #----------------------------------------------------------------------------
 def snip_print(str,width = 75,print_full = 0,header = ''):
     """Print a string snipping the midsection to fit in width.
@@ -1647,7 +1792,7 @@ def uniq_stable(elems):
     Note: All elements in the input must be valid dictionary keys for this
     routine to work, as it internally uses a dictionary for efficiency
     reasons."""
-    
+
     unique = []
     unique_dict = {}
     for nn in elems:
@@ -1662,13 +1807,13 @@ class NLprinter:
 
     An instance of this class called nlprint is available and callable as a
     function.
-    
+
     nlprint(list,indent=' ',sep=': ') -> prints indenting each level by 'indent'
     and using 'sep' to separate the index from the value. """
 
     def __init__(self):
         self.depth = 0
-    
+
     def __call__(self,lst,pos='',**kw):
         """Prints the nested list numbering levels."""
         kw.setdefault('indent',' ')
@@ -1681,7 +1826,7 @@ class NLprinter:
         stop = kw['stop']; del kw['stop']
         if self.depth == 0 and 'header' in kw.keys():
             print kw['header']
-            
+
         for idx in range(start,stop):
             elem = lst[idx]
             if type(elem)==type([]):
@@ -1712,20 +1857,6 @@ def sort_compare(lst1,lst2,inplace = 1):
     lst1.sort(); lst2.sort()
     return lst1 == lst2
 
-#----------------------------------------------------------------------------
-def mkdict(**kwargs):
-    """Return a dict from a keyword list.
-
-    It's just syntactic sugar for making ditcionary creation more convenient:
-    # the standard way
-    >>>data = { 'red' : 1, 'green' : 2, 'blue' : 3 }
-    # a cleaner way
-    >>>data = dict(red=1, green=2, blue=3)
-
-    If you need more than this, look at the Struct() class."""
-
-    return kwargs
-
 #----------------------------------------------------------------------------
 def list2dict(lst):
     """Takes a list of (key,value) pairs and turns it into a dict."""
@@ -1844,7 +1975,7 @@ def getattr_list(obj,alist,*args):
             raise ValueError,'getattr_list() takes only one optional argument'
     else:
         return map(lambda attr: getattr(obj,attr),alist)
-    
+
 #----------------------------------------------------------------------------
 def map_method(method,object_list,*argseq,**kw):
     """map_method(method,object_list,*args,**kw) -> list
@@ -1896,7 +2027,7 @@ def dir2(obj):
     are later not really valid for attribute access (many extension libraries
     have such bugs).
     """
-    
+
     # Start building the attribute list via dir(), and then complete it
     # with a few extra special-purpose calls.
     words = dir(obj)
@@ -1973,12 +2104,61 @@ def popkey(dct,key,default=NotGiven):
 
 def wrap_deprecated(func, suggest = '<nothing>'):
     def newFunc(*args, **kwargs):
-        warnings.warn("Call to deprecated function %s, use %s instead" % 
+        warnings.warn("Call to deprecated function %s, use %s instead" %
                       ( func.__name__, suggest),
                       category=DeprecationWarning,
                       stacklevel = 2)
         return func(*args, **kwargs)
     return newFunc
-    
-#*************************** end of file <genutils.py> **********************
 
+
+def _num_cpus_unix():
+    """Return the number of active CPUs on a Unix system."""
+    return os.sysconf("SC_NPROCESSORS_ONLN")
+
+
+def _num_cpus_darwin():
+    """Return the number of active CPUs on a Darwin system."""
+    p = subprocess.Popen(['sysctl','-n','hw.ncpu'],stdout=subprocess.PIPE)
+    return p.stdout.read()
+
+
+def _num_cpus_windows():
+    """Return the number of active CPUs on a Windows system."""
+    return os.environ.get("NUMBER_OF_PROCESSORS")
+
+
+def num_cpus():
+   """Return the effective number of CPUs in the system as an integer.
+
+   This cross-platform function makes an attempt at finding the total number of
+   available CPUs in the system, as returned by various underlying system and
+   python calls.
+
+   If it can't find a sensible answer, it returns 1 (though an error *may* make
+   it return a large positive number that's actually incorrect).
+   """
+
+   # Many thanks to the Parallel Python project (http://www.parallelpython.com)
+   # for the names of the keys we needed to look up for this function.  This
+   # code was inspired by their equivalent function.
+
+   ncpufuncs = {'Linux':_num_cpus_unix,
+                'Darwin':_num_cpus_darwin,
+                'Windows':_num_cpus_windows,
+                # On Vista, python < 2.5.2 has a bug and returns 'Microsoft'
+                # See http://bugs.python.org/issue1082 for details.
+                'Microsoft':_num_cpus_windows,
+                }
+
+   ncpufunc = ncpufuncs.get(platform.system(),
+                            # default to unix version (Solaris, AIX, etc)
+                            _num_cpus_unix)
+
+   try:
+       ncpus = max(1,int(ncpufunc()))
+   except:
+       ncpus = 1
+   return ncpus
+
+#*************************** end of file <genutils.py> **********************
